
%% REPRODUCING EXPERIMENT 6.1
clear; % we need to clear the variables in workspace before learning.
close all
mex ClocalNormalizeFloat.c; 
HV = 5; LV = 7; 
for (di = 1:HV)
vrarrayAll(di, :) = -(di-1)*(0.1:0.05:0.4);
vcarrayAll(di, :) = -10*(0.1:0.05:0.4);
end
bx0 = 114; by0 = 102; bx1 = 226; by1 = 244;

%% CONSTANTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
scale = 0.7;  % scale of Gabors, length = 17 pixels default =.7
norient = 15;  % number of orientations
epsilon = .1;  % allowed correlation between selected Gabors
Totalsketch = 60;  % allowed number of selected Gabors   %default 60,
Upperbound = 16.; % saturation value
sub = 2;    % subsampling rate for Gabor
Lrange = 3; % allowed shifting in location (times sub pixels)
Orange = 1; % allowed shifting in orientation
%%% Note: Lrange*sub is the pixels allowed to shift along normal
%%%       Lrange*sub should be the same in learning and testing
resizescale = 1.8;
binnum = 50; % number of bins 
LIK = 0; SHUTUP = 1.;  % If LIK=0, then use active correlation
% SHUTUP is to neglect the positions with low average responses
localHalfx = 20; localHalfy = 20; % the half range for local normalization, has to be quite large
thresholdFactor = .01;  % divide the response by max(average, maxAverage*thresholdFactor)
                       
%% TRAINING EXAMPLES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Folder = 'sequence';  
Iname = dir([Folder '\*.jpg']);
nimage = size(Iname, 1);
Isize = zeros(nimage, 2);
for (i = 1 : nimage)
    I0 = single(rgb2gray(imread([Folder '\' Iname(i).name]))); 
    I1 = I0(bx0:bx1, by0:by1); 
    I{i} = imresize(I1, resizescale, 'nearest');
    Isize(i, :) = size(I{i});
end
sx = min(Isize(:, 1)); sy = min(Isize(:, 2)); 

%% FILTERING IMAGES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp(['start filtering']);
tic
[allfilter, allsymbol] = makefilter_float(scale, norient);  % generate Gabor filters 
h = (size(allfilter{1}, 1)-1)/2;  % half size of Gabor
C = corr_float(allfilter, single(epsilon));    % generate the inhibition maps
allfiltered = applyfilterfftsame(I, allfilter, localHalfx, localHalfy, thresholdFactor);  % filter training images
%---Background model, subtract average responses of all images
allfiltered = backgroundSubtraction(allfiltered, 0); %ave num of occl fr
save allfiltered allfiltered 
disp(['filtering time: ' num2str(toc) ' seconds']);

%% VARIABLES FOR MEX-C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Mi =  (zeros(1, Totalsketch));  % orientation of selected Gabor
Mx =  (zeros(1, Totalsketch));  % position of selected Gabor
My =  (zeros(1, Totalsketch)); 
Mm =  (zeros(1, Totalsketch));  % average response of selected Gabor
alignframe = round(nimage/2);
sym1 =  single(zeros(sx, sy));  % symbolic plot of selected Gabor, Average
Asym = cell(1, nimage);   % symbolic plot for each image with active Gabor, Each ONE
for i = 1 : nimage
    Asym{i} = single(zeros(sx, sy));  
end
tss =  (zeros(nimage, 1));  % total scores for testing images
AsymAll = cell(HV, LV, nimage); 
sym1All = cell(HV, LV);
tssSumAll = zeros(HV, LV); 
tssSumMax = zeros(HV, 1); 

%% Begin Loop Enumarating Velocities %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
mex Clearn_float.c;   % compile C code
for di = 1:HV
    disp(['-----> learning on direction number ' num2str(di)]);
for vid = 1:LV
load allfiltered
vy = vcarrayAll(di, vid)*resizescale;
vx = vrarrayAll(di, vid)*resizescale;
disp(['learning on speed number ' num2str(vid)]);
tic
Clearn_float(nimage, norient, allfiltered, vx, vy, alignframe, ... 
    C, h, sx, sy, Mi, Mx, My, Mm, allsymbol(1, :), sym1, Asym,...
    Lrange, Orange, sub, Totalsketch, Upperbound, tss, ...
    LIK, SHUTUP, binnum);
disp(['mex-C learning time: ' num2str(toc) ' seconds']);
sym1All{di, vid} = sym1+0.; 
tssSumAll(di, vid) = sum(tss);
for (i=1:nimage)
    AsymAll{di, vid, i} = Asym{i}+0.;
end
end
tssSumMax(di) = max(tssSumAll(di, :)); 
end

%% RESULTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    [tssM, di] = max(tssSumMax); 
    figure;
    plot(vcarrayAll(di, :), tssSumAll(di, :), 'LineWidth', 2, 'Marker', '*');
    title('Alignment score vs Velocity');
    xlabel('Velocity');
    ylabel('Score');
    [maxtt, vid] = max(tssSumAll(di, :));
    
    for j = 1:LV
       outepsgif(-sym1All{di, j}, ['LNex6_man_templates' num2str(j)]);  
    end        
    vy = vcarrayAll(di, vid)*resizescale;
    vx = vrarrayAll(di, vid)*resizescale;

mkdir('togif'); 
K = nimage; bx =0; by = 0; cc = 3;  
col = 5; row = floor(K/col)+1;
Iout = zeros(row*sx+(row-1)*(2*bx+cc), col*sy+(col-1)*(2*by+cc))+255; 
towrite =  -sym1All{di, vid}; 
towrite = (255 * (towrite-min(towrite(:)))/(max(towrite(:))-min(towrite(:))));
Iout(1:sx, 1:sy) = towrite;
i = -1;  
for (r = 1 : row)
    for (c = 1 : col)
        i = i+1; 
        if ((i>=1)&&(i<=K))            
            symshow = shift_image(AsymAll{di, vid, i}, vx*(i-alignframe), vy*(i-alignframe)); 
            towrite =  I{i}+symshow*64; 
            towrite = (255 * (towrite-min(towrite(:)))/(max(towrite(:))-min(towrite(:))));
            Iout((r-1)*sx+1+(r-1)*(2*bx+cc): r*sx+(r-1)*(2*bx+cc), (c-1)*sy+1+(c-1)*(2*by+cc): c*sy+(c-1)*(2*by+cc)) = towrite; 
            towrite = uint8(towrite); 
           imwrite(towrite,['togif' '\sym' num2str(i) '.jpg'],'jpeg');          
        end
    end
end

outepsgif(Iout, 'LNex6_man_sequence'); 
    
    